#include <SPI.h>
#include <Ethernet.h>
#include <StopWatch.h>

#define DEBUG

// Enter a MAC address and IP address for your controller below.
// The IP address will be dependent on your local network, and it's optional if DHCP is enabled
byte mac[] = { 0x90, 0xA2, 0xDA, 0x0D, 0xBC, 0xAE };
IPAddress ip(192,168,15,177);
static const byte deviceId[] = { 0x00, 0x00, 0x00, 0x01 };
static const uint8_t ACK = 0x01;
static const int LED_PIN = 8;
int connected = 0;
EthernetClient client;
StopWatch stopWatch;
long pingInterval = 200000;


void setup() 
{
    Serial.begin(9600);
    Serial.println("Initialized");
    //Ethernet.begin(mac);
    Ethernet.begin(mac, ip);
    pinMode(LED_PIN, OUTPUT);
}

void turnLedOn()
{
    digitalWrite(LED_PIN, HIGH);
}

void turnLedOff()
{
    digitalWrite(LED_PIN, LOW);
}

void loop() 
{      
        if ( connected == 0)
	{
                Serial.println("Trying to connect");
        	char* host = "msdnlistener.cloudapp.net";
		//char* host = "192.168.15.100";

		client.setTimeout(10000);
		connected = client.connect(host, 10100);
		if (connected) 
		{
		    Serial.println("Connected to port, writing deviceId and waiting for commands...");
		    client.write(deviceId, sizeof(deviceId));
                    stopWatch.start();
		}
                else
                {
                    Serial.println("Connection unsuccessful");
                    client.stop();
                    stopWatch.reset();
                }
	}

	if (connected == 1)
	{
                if (stopWatch.elapsed() > pingInterval)
                {
                   Serial.println("Pinging Server to keep connection alive...");
                   client.write(deviceId, sizeof(deviceId));
                   stopWatch.reset(); 
                   stopWatch.start();
                }
                
		byte buf[16];
		int readResult = client.read(buf, 1);
                
		if (readResult == 0) 
		{
                    Serial.println("Can't find listener, disconnecting...");
		    connected = 0;
                    stopWatch.reset();
		}
		else if (readResult == 1)
		{
			Serial.println("Data acquired, processing...");
			switch ( buf[0] )
			{
			case 1:
				// turn LED on
                                Serial.println("Command to turn led on received...");
				turnLedOn();
				break;
			case 2: 
				// turn LED off 
                                Serial.println("Command to turn led off received...");
				turnLedOff();
				break;
			}
                          
                        stopWatch.reset();
                        stopWatch.start();
		}
	}
}
