﻿'''**************************** Module Header ******************************\
'''* Module Name:    MainPage.xaml.vb
'''* Project:        MSDNArduinoClient
'''* 
'''* This source is subject to the Microsoft Public License.
'''* See http://www.microsoft.com/en-us/openness/licenses.aspx#MPL.
'''* All other rights reserved.
'''* 
'''* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
'''* EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
'''* WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
'''\***
''' *************************************************************************
Imports System.IO
Imports System.Linq
Imports System.Text
Imports System.Net
Imports System.Windows
Imports System.Windows.Controls
Imports System.Windows.Navigation
Imports Microsoft.Phone.Controls
Imports Microsoft.Phone.Shell
Imports MSDNArduinoClient.Resources
Imports System.Threading.Tasks
Imports Microsoft.WindowsAzure.MobileServices
Imports Microsoft.VisualBasic

Namespace MSDNArduinoClient
	Partial Public Class MainPage
		Inherits PhoneApplicationPage
		' Your Service Namespace
		Private Shared serviceNamespace As String = "msdnarduinoservicebus"
		Private Shared baseAddress As String = String.Empty
		Private Shared token As String = String.Empty
		Private Const sbHostName As String = "servicebus.windows.net"
		Private Const acsHostName As String = "accesscontrol.windows.net"

		' Your issuer name
		Private Shared issuerName As String = "owner"

		' Your issuer secret
		Private Shared issuerSecret As String = "qWjqKJFfJwB5JEkp5eI7ZfePYkrq6tnvFvn8X8hAIRc="

		' Note that the realm used when requesting a token uses the HTTP scheme, even though
		' calls to the service are always issued over HTTPS
		Private Shared realm As String = String.Empty

		' Constructor
		Public Sub New()
			InitializeComponent()
		End Sub

		''' <summary>
		''' Generate Token.
		''' </summary>
		''' <param name="url"></param>
		''' <param name="postdata"></param>      
		''' <returns></returns>
		Private async Function post(ByVal url As String, ByVal postdata As String) As Task(Of String)
			Dim request = TryCast(WebRequest.Create(New Uri(url)), HttpWebRequest)
			request.ContentType = "application/x-www-form-urlencoded"
			request.Method = "POST"

			Dim data() As Byte = Encoding.UTF8.GetBytes(postdata)
			request.ContentLength = data.Length

            Using requestStream = Await Task(Of Stream).Factory.FromAsync(AddressOf request.BeginGetRequestStream, AddressOf request.EndGetRequestStream, request)
                Await requestStream.WriteAsync(data, 0, data.Length)
            End Using

            Dim responseObject As WebResponse = Await Task(Of WebResponse).Factory.FromAsync(AddressOf request.BeginGetResponse, AddressOf request.EndGetResponse, request)
            Dim responseStream = responseObject.GetResponseStream
			Dim sr = New StreamReader(responseStream)
            Dim received As String = Await sr.ReadToEndAsync

			Return received
		End Function

		''' <summary>
		''' Create Queue.
		''' </summary>
		''' <param name="queueName"></param>
		''' <param name="token"></param>
		''' <returns></returns>
		Private Sub CreateQueue(ByVal queueName As String, ByVal token As String)
			' Create the URI of the new Queue, note that this uses the HTTPS scheme
			Dim queueAddress = baseAddress & queueName
            Dim webClient As New WebClient
			webClient.Headers(HttpRequestHeader.Authorization) = token

			' Prepare the body of the create queue request
            Dim putData = "<entry xmlns=""http://www.w3.org/2005/Atom"">" & vbCrLf & "                                          <title type=""text"">" & queueName & "</title>" & vbCrLf & "                                          <content type=""application/xml"">" & vbCrLf & "                                            <QueueDescription xmlns:i=""http://www.w3.org/2001/XMLSchema-instance"" xmlns=""http://schemas.microsoft.com/netservices/2010/10/servicebus/connect"" />" & vbCrLf & "                                          </content>" & vbCrLf & "                                        </entry>"

            AddHandler webClient.UploadStringCompleted, (Sub(sender, e)
                                                             If e.Error IsNot Nothing Then
                                                                 Exit Sub
                                                             End If
                                                             Debug.WriteLine(vbCrLf & "Creating queue {0}", queueAddress)
                                                             Debug.WriteLine(e.Result)
                                                         End Sub)

			webClient.UploadStringAsync(New Uri(queueAddress), "PUT", putData, token)
		End Sub

		''' <summary>
		''' Send Message.
		''' </summary>
		''' <param name="relativeAddress"></param>
		''' <param name="body"></param>
		Private Sub SendMessage(ByVal relativeAddress As String, ByVal body As String)
			Dim fullAddress As String = baseAddress & relativeAddress & "/messages" & "?timeout=60"

            Dim webClient As New WebClient
			webClient.Headers(HttpRequestHeader.Authorization) = token

            AddHandler webClient.UploadStringCompleted, (Sub(sender, e)
                                                             Debug.WriteLine(vbCrLf & "Sending message {0} - to address {1}", body, fullAddress)
                                                             If e.Error IsNot Nothing Then
                                                                 Exit Sub
                                                             End If
                                                             Debug.WriteLine(e.Result)
                                                         End Sub)

			webClient.UploadStringAsync(New Uri(fullAddress), "POST", body, token)
		End Sub

		''' <summary>
		''' Receive Message.
		''' </summary>
		''' <param name="relativeAddress"></param>
		''' <returns></returns>
		Private Sub ReceiveMessage(ByVal relativeAddress As String)
			Dim fullAddress As String = baseAddress & relativeAddress & "/messages/head" & "?timeout=60"
            DownloadData(fullAddress, String.Format(vbCrLf & "Retrieving message from {0}", fullAddress))
		End Sub



		''' <summary>
		''' Download Data.
		''' </summary>
		''' <param name="fullAddress">Uri string.</param>
		''' <param name="strMessage">Tip message.</param>
		Private Shared Sub DownloadData(ByVal fullAddress As String, ByVal strMessage As String)
            Dim webClient As New WebClient
			webClient.Headers(HttpRequestHeader.Authorization) = token

            AddHandler webClient.DownloadStringCompleted, (Sub(sender, e)
                                                               Debug.WriteLine(strMessage)
                                                               If e.Error IsNot Nothing Then
                                                                   Exit Sub
                                                               End If
                                                               Debug.WriteLine(e.Result)
                                                           End Sub)
			webClient.DownloadStringAsync(New Uri(fullAddress))
		End Sub

		Public async Function CallMobileServerAsync(ByVal command As String) As Task
			baseAddress = "https://" & serviceNamespace & "." & sbHostName & "/"
			Try
				Dim acsEndpoint As String = "https://" & serviceNamespace & "-sb." & acsHostName & "/WRAPv0.9/"
				realm = "http://" & serviceNamespace & "." & sbHostName & "/"

				Dim postdata As String = "wrap_name=" & Uri.EscapeDataString(issuerName) & "&wrap_password=" & Uri.EscapeDataString(issuerSecret) & "&wrap_scope=" & Uri.EscapeDataString(realm)

				Dim strResponse As String = await post(acsEndpoint, postdata)
				Dim responseProperties = strResponse.Split("&"c)
				Dim tokenProperty = responseProperties(0).Split("="c)
				token = Uri.UnescapeDataString(tokenProperty(1))
				token = "WRAP access_token=""" & token & """"

				Dim queueName As String = txtDeviceName.Text
				SendMessage(queueName, command)
			Catch we As WebException
				Using response As HttpWebResponse = TryCast(we.Response, HttpWebResponse)
					If response IsNot Nothing Then
                        Debug.WriteLine(New StreamReader(response.GetResponseStream).ReadToEnd)
					Else
                        Debug.WriteLine(we.ToString)
					End If
				End Using
			End Try
		End Function

		Private async Sub ButtonOn_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			await Me.CallMobileServerAsync("ON")
		End Sub

		Private async Sub ButtonOff_Click(ByVal sender As Object, ByVal e As RoutedEventArgs)
			await Me.CallMobileServerAsync("OFF")
		End Sub

	End Class


End Namespace