﻿'''**************************** Module Header ******************************\
'''* Module Name:    Program.vb
'''* Project:        MSDNArduinoListener
'''* 
'''* This source is subject to the Microsoft Public License.
'''* See http://www.microsoft.com/en-us/openness/licenses.aspx#MPL.
'''* All other rights reserved.
'''* 
'''* THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND, 
'''* EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE IMPLIED 
'''* WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR PURPOSE.
'''\****************************************************************************

Imports System.Globalization
Imports System.IO
Imports System.Net
Imports System.Net.Sockets
Imports System.Text
Imports System.Threading
Imports System.Threading.Tasks
Imports Microsoft.WindowsAzure
Imports Microsoft.ServiceBus
Imports Microsoft.ServiceBus.Messaging

Namespace ConsoleListener
	Friend Class Program
		Private Shared timeout As Integer = 10000
		Private Shared ReadOnly OnFrame() As Byte = { &H1 }
		Private Shared ReadOnly OffFrame() As Byte = { &H2 }

		Shared Sub Main(ByVal args() As String)
			Console.WriteLine("Starting Application")
			Trace.WriteLine("Starting Application", "Information")

			Dim serviceBusConnectionString As String = "Endpoint=sb://msdnarduinoservicebus.servicebus.windows.net/;SharedSecretIssuer=owner;SharedSecretValue=qWjqKJFfJwB5JEkp5eI7ZfePYkrq6tnvFvn8X8hAIRc="
			Dim deviceEp As IPEndPoint = CreateIPEndPoint("192.168.15.100:10100")
            Dim messagingFactoryVar = MessagingFactory.CreateFromConnectionString(serviceBusConnectionString)

            RunServer(deviceEp, messagingFactoryVar)
		End Sub

		Private Shared Sub RunServer(ByVal deviceEndPoint As IPEndPoint, ByVal messagingFactory As MessagingFactory)
			Console.WriteLine("Starting Server")
			Task.WaitAll( {RunDeviceEndpointAsync(deviceEndPoint, messagingFactory)})
		End Sub

		Private Shared async Function RunDeviceEndpointAsync(ByVal deviceEP As IPEndPoint, ByVal messagingFactory As MessagingFactory) As Task
			Console.WriteLine("Listening for connections")
			Dim deviceServer = New TcpListener(deviceEP)
			deviceServer.Start(10)

			Try
				Do
                    Dim connection As TcpClient = Await deviceServer.AcceptTcpClientAsync
					If connection IsNot Nothing Then
						Try
							connection.NoDelay = True ' flush writes immediately to the wire
							connection.ReceiveTimeout = timeout
                            Dim deviceConnectionStream As NetworkStream = connection.GetStream
							Dim readBuffer = New Byte(63){}

							If await deviceConnectionStream.ReadAsync(readBuffer, 0, 4) = 4 Then
								Dim deviceId As Integer = IPAddress.NetworkToHostOrder(BitConverter.ToInt32(readBuffer, 0))
								Console.WriteLine("Received signal for device:" & deviceId)
								Trace.WriteLine("Received signal for device:" & deviceId)

								Try
									'Create device queue if it doesn't exist
                                    Dim namespaceManagerVar = NamespaceManager.CreateFromConnectionString(CloudConfigurationManager.GetSetting("serviceBusConnectionString"))
                                    If Not namespaceManagerVar.QueueExists(String.Format("dev{0:X8}", deviceId)) Then
                                        namespaceManagerVar.CreateQueue(String.Format("dev{0:X8}", deviceId))
                                    End If

									Console.WriteLine(String.Format("dev{0:X8}", deviceId))
									' set up the receiver for the per-device queue and sender for the response queue
									Dim deviceQueueReceiver = messagingFactory.CreateMessageReceiver(String.Format("dev{0:X8}", deviceId), ReceiveMode.PeekLock)

									Console.WriteLine("Connected to Queue")

									Do
										' receive from the queue
										Console.WriteLine("Reading the Queue")
										Trace.WriteLine("Reading the Queue")
										Dim message As BrokeredMessage = Nothing
                                        message = Await deviceQueueReceiver.ReceiveAsync

										Try
											If message IsNot Nothing Then
												Dim stream As Stream = message.GetBody(Of Stream)()
												Dim reader As New StreamReader(stream)
                                                Dim command As String = reader.ReadToEnd

												If command IsNot Nothing Then
                                                    Select Case command.ToUpperInvariant
                                                        Case "ON"
                                                            Await deviceConnectionStream.WriteAsync(OnFrame, 0, OnFrame.Length)
                                                            Await message.CompleteAsync
                                                        Case "OFF"
                                                            Await deviceConnectionStream.WriteAsync(OffFrame, 0, OffFrame.Length)
                                                            Await message.CompleteAsync
                                                    End Select
												End If
											End If
										Catch e As Exception
											Console.WriteLine(e.Message)
											deviceQueueReceiver.Close()
										End Try
									Loop While True
								Catch e As Exception
									Console.WriteLine(e.Message)
								Finally
									connection.Close()
								End Try
							Else
								connection.Close()
							End If
						Catch se As SocketException
							' log
							Console.WriteLine(se.Message)
							connection.Close()
						Catch ioe As IOException
							Console.WriteLine(ioe.Message)
							connection.Close()
						Catch e As Exception
							Console.WriteLine(e.Message)
						End Try
					End If
				Loop While True
			Catch e As Exception
				Trace.TraceError(e.Message)
			Finally
				deviceServer.Stop()
			End Try
		End Function

		Private Shared Function CreateIPEndPoint(ByVal endPoint As String) As IPEndPoint
			Dim ep() As String = endPoint.Split(":"c)
			If ep.Length <> 2 Then
				Throw New FormatException("Invalid endpoint format")
			End If
            Dim ip As IPAddress = New IPAddress(665567)
            If Not IPAddress.TryParse(ep(0), ip) Then
                Throw New FormatException("Invalid ip-adress")
            End If
			Dim port As Integer
			If Not Integer.TryParse(ep(1), NumberStyles.None, NumberFormatInfo.CurrentInfo, port) Then
				Throw New FormatException("Invalid port")
			End If
			Return New IPEndPoint(ip, port)
		End Function
	End Class
End Namespace
